/* client.cpp
 *
 * This is an ASCII client for the Borbaman game.
 */


/************
 * INCLUDES *
 ************/ 

// CORBA includes
#include <OB/CORBA.h>
#include <Game.h>

// standard includes
#include <stdlib.h>
#include <ncurses.h>

// project includes
#include "client.h"


/*************
 * CONSTANTS *
 *************/

// number of rows in the map
const unsigned int NUM_ROWS = 11;

// number of columns in the map
const unsigned int NUM_COLS = 15;

// row offset in the terminal to start displaying the map
const unsigned int MAP_ROW_OFFSET = 5;

// column offset in the terminal to start displaying the map
const unsigned int MAP_COL_OFFSET = 10;

// colors for things on the map
const short CLR_BRICK = 1;


/***********
 * GLOBALS *
 ***********/

// intial map state
// 0 means empty space
// 1 means un-bombable wall
// 2 means bombable brick
const int start_map[NUM_ROWS][NUM_COLS] = {{0,0,2,2,2,2,2,2,2,2,2,2,2,0,0},
                                           {0,1,2,1,2,1,2,1,2,1,2,1,2,1,0},
                                           {2,2,2,2,2,2,2,2,2,2,2,2,2,2,2},
                                           {2,1,2,1,2,1,2,1,2,1,2,1,2,1,2},
                                           {2,2,2,2,2,2,2,2,2,2,2,2,2,2,2},
                                           {2,1,2,1,2,1,2,1,2,1,2,1,2,1,2},
                                           {2,2,2,2,2,2,2,2,2,2,2,2,2,2,2},
                                           {2,1,2,1,2,1,2,1,2,1,2,1,2,1,2},
                                           {2,2,2,2,2,2,2,2,2,2,2,2,2,2,2},
                                           {0,1,2,1,2,1,2,1,2,1,2,1,2,1,0},
                                           {0,0,2,2,2,2,2,2,2,2,2,2,2,0,0}};

// current map state
int map[NUM_ROWS][NUM_COLS];


/*************
 * FUNCTIONS *
 *************/

bool initialize() {
  // puts the client into curses mode
  initscr();

  // enable colors
  if (start_color() == ERR)
    return true;

  // prevents characters from being echoed to the screen as the user types
  if (noecho() == ERR)
    return true;

  // don't buffer key input
  if (cbreak() == ERR)
    return true;

  // enable extended keys like the arrow keys and function keys
  if (keypad(stdscr, TRUE) == ERR)
    return true;

  // make the cursor invisible
  if (curs_set(0) == ERR)
    return true;

  // initialize the map state
  for (unsigned int i = 0; i < NUM_ROWS; i++) {
    for (unsigned int j = 0; j < NUM_COLS; j++) {
      map[i][j] = start_map[i][j];
    }
  }

  // create the game colors
  init_pair(CLR_BRICK, COLOR_YELLOW, COLOR_BLACK);

  return false;
}

void cleanup() {
  // restores the terminal to the original mode
  endwin();
}

void draw_map() {
  // draw the map border
  draw_map_border();

  // draw the map contents
  for (unsigned int row = 0; row < NUM_ROWS; row++) {
    for (unsigned int col = 0; col < NUM_COLS; col++) {
      switch (map[row][col]) {
        case 1:   // wall
          attron(A_REVERSE);
          mvaddch(row+MAP_ROW_OFFSET+1, col+MAP_COL_OFFSET+1, '#');
          attroff(A_REVERSE);
          break;
        case 2:   // bombable brick
          attron(COLOR_PAIR(CLR_BRICK));
          mvaddch(row+MAP_ROW_OFFSET+1, col+MAP_COL_OFFSET+1, ACS_CKBOARD);
          attroff(COLOR_PAIR(CLR_BRICK));
          break;
      }
    }
  }
}

void draw_map_border() {
  attron(A_BOLD);

  // draw the top and bottom borders
  for (unsigned int col = 1; col < NUM_COLS+1; col++) {
    mvaddch(MAP_ROW_OFFSET, col + MAP_COL_OFFSET, ACS_HLINE);
    mvaddch(MAP_ROW_OFFSET + NUM_ROWS + 1, col + MAP_COL_OFFSET, ACS_HLINE);
  }
  
  // draw the left and right borders
  for (unsigned int row = 1; row < NUM_ROWS+1; row++) {
    mvaddch(row + MAP_ROW_OFFSET, MAP_COL_OFFSET, ACS_VLINE);
    mvaddch(row + MAP_ROW_OFFSET, NUM_COLS + MAP_COL_OFFSET + 1, ACS_VLINE);
  }

  // draw the corners
  mvaddch(MAP_ROW_OFFSET, MAP_COL_OFFSET, ACS_ULCORNER);
  mvaddch(MAP_ROW_OFFSET+NUM_ROWS+1, MAP_COL_OFFSET, ACS_LLCORNER);
  mvaddch(MAP_ROW_OFFSET, MAP_COL_OFFSET+NUM_COLS+1, ACS_URCORNER);
  mvaddch(MAP_ROW_OFFSET+NUM_ROWS+1, MAP_COL_OFFSET+NUM_COLS+1, ACS_LRCORNER);

  attroff(A_BOLD);
}


/********
 * MAIN *
 ********/

int client_main(int argc, char *argv[], Game_var game) {
  // initialize the client 
  if (initialize()) {
    printf("Error occurred during initialization");
    cleanup();
    return -1;
  }

  mvprintw(20, 0, "press the enter key to exit");

  // refresh the map display on the screen
  draw_map();

  // need to call refresh to update the display with the most recent changes
  refresh();

  // wait for user input
  int c;
  struct ClientAction action;
  struct GameState gamestate;
  while ((c = getch()) != '\n') {
    switch (c) {
      case KEY_LEFT:
        action.arrow = MOVE_LEFT;
        gamestate = game->update(action);
        mvprintw(0, 0, "Received %d from the server", gamestate.errval);
        break;
      case KEY_RIGHT:
        action.arrow = MOVE_RIGHT;
        gamestate = game->update(action);
        mvprintw(0, 0, "Received %d from the server", gamestate.errval);
        break;
      case KEY_UP:
        action.arrow = MOVE_UP;
        gamestate = game->update(action);
        mvprintw(0, 0, "Received %d from the server", gamestate.errval);
        break;
      case KEY_DOWN:
        action.arrow = MOVE_DOWN;
        gamestate = game->update(action);
        mvprintw(0, 0, "Received %d from the server", gamestate.errval);
        break;
    }
  }

  // perform all cleanup
  cleanup();

  return 0;
}

